#!/usr/bin/env python

"""
Examples of the use of the EaterWorld class
"""

import random
from typing import Iterable
import sys

from wumpus import OfflinePlayer, run_episode, Eater, EaterWorld, Food


class GooPlayer(OfflinePlayer):
    """
    Offline player demonstrating the use of the start episode method to inspect the world.
    """

    def _say(self, text: str):
        print(self.name + ' says: ' + text)

    def start_episode(self, world: EaterWorld) -> Iterable[Eater.Actions]:
        """
        Print the description of the world before starting.
        """
        # keep track of the reward
        self.reward = 0

        self._say('Episode starting for player {}'.format(self.name))

        # inspect the objects in the world
        food_locations = []
        eater_location = None
        for o in world.objects:
            if isinstance(o, Eater):
                eater_location = (o.location.x, o.location.y)
                all_actions = list(o.Actions)
            elif isinstance(o, Food):
                food_locations.append((o.location.x, o.location.y))

        # get the list of blocks
        block_locations = sorted((bl.x, bl.y) for bl in world.blocks)

        # Print the description of the world
        self._say('World size: {}x{}'.format(world.size.x, world.size.y))
        self._say('Eater agent in {}'.format(eater_location))
        self._say('Food in {}'.format(sorted(food_locations)))
        self._say('Blocks in {}'.format(block_locations))

        self._say('Available actions: {}'.format({a.name: a.value for a in all_actions}))

        # creates an iterator that returns a sequence of random actions
        def random_actions():
            # prevent unbounded iterations
            for _ in range(10000):
                yield all_actions[random.randint(0, len(all_actions) - 1)]

        return random_actions()


    def end_episode(self, outcome: int, alive: bool, success: bool):
        """Method called at the when an episode is completed."""
        self._say('Episode completed, my reward is {}'.format(outcome))


MAP_STR = """
################
#    #    #    #
#    #    #    #
#    #         #
#    #    #    #
###           ##
"""


def main(*args):
    """
    Play a random EaterWorld episode using the default player
    """

    player_class = GooPlayer

    world = EaterWorld.random(map_desc=MAP_STR)
    player = player_class('Hungry.Monkey')

    run_episode(world, player, horizon=20)

    return 0


if __name__ == "__main__":
    sys.exit(main(*sys.argv[1:]))
